# -*- coding: utf-8 -*-
"""
Created on 7.4. 2021
PI-Regler mit AWR und Strecke mit ioSys
@author: philippsen
"""
import control as ct
import numpy as np
import matplotlib.pyplot as plt
plt.rcParams['pdf.fonttype'] = 42
plt.rcParams['ps.fonttype'] = 42

def pi_update(t, x, u, params={}):
    # Reglerparameter übergeben, falls erforderlich
    kp = params.get('kp', 0.5)
    Tn = params.get('Tn', 1.0)
    ki = kp/Tn
    Tikor =  0.9*Tn  # Anti-Windup-Korrekturfaktor
    ybeg = params.get('ybeg', 2.0)

    # Zuweisung von Variablen und Zuständen
    e = u          #  Regelfehler
    ei = x[0]      # Zustand korr. integrierter Regelfehler
    # Berechnete Stellgröße ohne Begrenzung
    y_a = pi_output(t, x, u, params)
    # Berechnung Anti-Windup-Kompensation 
    e_kor = (np.clip(y_a, -ybeg, ybeg) - y_a)/(ki*Tikor) if Tn != 0 else 0

    return e + e_kor # eiPunkt = e + e_kor

def pi_output(t, x, u, params={}):
    # Reglerparameter übergeben, falls erforderlich
    kp = params.get('kp', 0.5)
    Tn = params.get('Tn', 1.0)
    ybeg = params.get('ybeg', 2.0)
    ki = kp/Tn
    # Zuweisung von Variablen und Zuständen
    e =   u          # Regelfehler
    ei = x[0]        # integrierter korr. Regelfehler
    # PI Reglerstellgröße mit AWR
    return kp * e +  ki*ei

io_PI_awr = ct.NonlinearIOSystem(
    pi_update, pi_output, name='control',
    inputs = ['e'], outputs = ['yy'], states = ['ei'],
    params = {'kp':0.5, 'Tn':1.4, 'ybeg':10.0})
# Leider wird nicht die begrenzte Stellgröße berechnet,
# sondern die Unbegrenzte! Also nochmal begrenzen

def y_output(t, x, u, params={}):
    # parameter übergeben, falls erforderlich
    ybeg = params.get('ybeg', 2.0)
    # Zuweisung von Variablen und Zuständen
    yy = u[0]
    # nur Begrenzung
    return np.clip(yy, -ybeg, ybeg)

Begrenzung = ct.NonlinearIOSystem(
    None, y_output, name='begrenzung',
    inputs = ['yy'], outputs = ['yb'],
    params = {'ybeg':10.0}) 

X0 = [0, 0, 0, 0,0] # Anfangswerte
T = np.arange(0.0, 10.0, 0.05) # Simulationsdauer 
# Simulation Regler-Sprungantwort

io_PIawr = ct.interconnect([io_PI_awr, Begrenzung], inplist=['e'], outlist='yb')

ww = np.zeros([len(T)])
for i in range(len(T)):
    if T[i] > 1:
        ww[i] = 35
    else:
        ww[i] = 0  

# Strecke
strecke = ct.tf2io(ct.tf(5, [0.5, 1.5, 1]), inputs='yb', outputs='xs')
sumblk = ct.summing_junction(inputs=['r', '-xs'], output='e')
#Regelkreis aufbauen
Rk = ct.interconnect([strecke,
    io_PI_awr,Begrenzung, sumblk], inplist='r', outlist=['xs','yb'])

t, yy = ct.input_output_response(Rk, T, ww)
#
# Plot Regelkreis-Sprungantwort
plt.figure(1)
plt.plot(t,yy[0,:],t,yy[1,:])
plt.xlabel('t [s]');plt.ylabel('y (rot) , x (blau)')
plt.title('Sprungantwort (t=1)')
plt.grid()