# -*- coding: utf-8 -*-
"""

Standardregelstrecke mit PI-Regler, die Regler-Parameter werden in einem
Wertebereich variert (Batch-Betrieb) und die Güte gemäß ITAE-Kriterium 
in einem 3D-Plot dargestellt. Die optimalen Parameter sind im Minimum zu finden

Created on 19.7. 2022
@author: philippsen
""" 

import numpy as np
import control
import control.matlab
import matplotlib.pyplot as plt
from matplotlib import cm
from mpl_toolkits.mplot3d import Axes3D

n = 300 # Anzahl Schritte
T0 = 0.1 # Abtastzeit
kk = 30  # Anzahl Parametersätze
xx = np.zeros(n)    # Regelgrößenvektor
u = np.zeros(n)     # Stellgrößenvektor
tk= np.arange(0,T0*n + T0,T0)    # k-Achse
MKp = np.zeros((kk,kk))
dKp_kk = 3/kk; dTn_kk = 6/kk
MTn = np.zeros((kk,kk))
Q = np.zeros((kk,kk)) # Güte-Matrix

# reg = control.tf(Kp*[Tn, 1], [Tn, 0])
stand = control.tf([1],[30,31, 10, 1]) # Standardregelstrecke
w = 1.0    #Sollwert
for ii in range(0, kk): # Liste von 0 bis kk-1
    for jj in range(0, kk):
        Kp = dKp_kk*(ii + 1); Tn = 5.0 + dTn_kk*(jj +1)      # Reglerparameter
        reg = control.tf([Kp*Tn, Kp], [Tn, 0])
        MKp[ii,jj] = Kp; MTn[ii,jj] = Tn
        G0 = reg*stand
        Gw = control.feedback(G0, 1) # % Regelkreis
        xk , tt = control.matlab.step(Gw,tk)
        e = w - xk
        e = e*tk
        Q[ii,jj] = sum( abs(e))*T0 #  Nachbildung Gütefunktion ITAE

print('Minimum = ',Q.min())
pa = np.where(Q == Q.min())
print('Kp = ',MKp[pa[0],pa[1]],'   Tn = ',MTn[pa[0],pa[1]] )
Kp = MKp[pa[0],pa[1]] ;    Tn = MTn[pa[0],pa[1]] 
Kp=Kp[0]; Tn=Tn[0]
R = control.tf([Kp*Tn, Kp], [Tn, 0])

fig = plt.figure()
ax = fig.add_subplot(projection='3d')

ax.plot_surface(MKp, MTn, Q, rstride=1, cstride=1, cmap=cm.viridis)
plt.xlabel('Kp'); plt.ylabel('Tn'); ax.set_zlabel('Q')
#fig2 = plt.figure()
Gw = control.feedback(R*stand,1)
stell = control.feedback(R, stand)
x, t = control.matlab.step(Gw)
y, t = control.matlab.step(stell,t)


fig, ax1 = plt.subplots()
ax1.plot(t, x, "b") 
ax1.set_ylabel('x', color="blue", fontsize=14) 
plt.grid()
ax2 = ax1.twinx() 
ax2.plot(t, y, "r") 
ax2.set_ylabel('y', color="red", fontsize=14) 
ax1.set_xlabel('t [s]', fontsize=12) 
plt.title('Entwurf gemäß Batch Opt')
