# -*- coding: utf-8 -*-
"""

P-T1 in Reihenschaltung mit I-Glied
Created on 15.4. 2019 and 7.4.2024
Neues Plot-Paradigma ab Version 0.10 für bode und nyquist
@author: philippsen
"""

import numpy as np
import control
from   control.matlab import *
import matplotlib.pyplot as plt

Kp= 2.0    #    Verstärkung
T1 = 0.2   #    Zeitkonstante

N1  = np.array([T1, 1])

G1 = tf(Kp,N1)  # P-T1
Gi = tf(1.0,[1.0, 0])  # Integrator
G4 = G1*Gi          # I-T1
time = np.arange(0.0, 1.0, 0.001)
x4, tt = control.matlab.step(G4,time) 
plt.figure()
plt.plot(tt, x4, "r") 
plt.ylabel('x4', color="red", fontsize=14) 
plt.xlabel('t [s]', fontsize=12) 
plt.title('I-T1 Sprungantwort', fontsize=12)
plt.grid()

plt.figure()
#bode(G4)    # Matlab - Bode
control.bode_plot(G4)

plt.figure()
real, imag, ww = control.matlab.nyquist(G4,plot=False) 
plt.plot(real,imag)
plt.title('Ortskurve I-T1 Matlab Nyquist')
plt.xlabel('Real'); plt.ylabel('Imag')
plt.grid()

plt.figure() # Ortskurve mit neuer Funktion
response = control.nyquist_response(G4) 
response.plot()
plt.title('Control Nyquist_response')

plt.figure() # Nur pos. Frequenzen
response = control.nyquist_response(G4,omega_limits=[0.1 ,100]) 
plt.plot(response.response.real,response.response.imag)
plt.title('Ortskurve I-T1 response limit, pos. Frequenzen')
plt.xlabel('Real'); plt.ylabel('Imag')
plt.grid()

# Eine andere Lösung mit dem SciPy - Paket
from scipy import signal
num = G4.num[0][0][:]   # Zugriff auf Zähler
den = G4.den[0][0][:]   # Zugriff auf Nenner
gg=signal.TransferFunction(num,den)
w, H = signal.freqresp(gg)
plt.figure()
plt.plot(H.real, H.imag, "b")
plt.title('Ortskurve I-T1 scipy')
#plt.plot(H.real, -H.imag, "r")
plt.grid()


