
# -*- coding: utf-8 -*-
"""
Symmetrisches Optimum PI-Reg
P-T1 in Reihenschaltung mit I-Glied
Created on 17.8. 2021
@author: philippsen
"""

import numpy as np
import control
from   control.matlab import *
import matplotlib.pyplot as plt
plt.rcParams['pdf.fonttype'] = 42
plt.rcParams['ps.fonttype'] = 42

Kp= 1.0    #    Verstärkung Strecke
T1 = 0.2   #    Zeitkonstante
G1 = tf(Kp,[T1, 1])  # P-T1
Gi = tf(1.0,[1.0, 0])  # Integrator
G = G1*Gi          # I-T1
#K = 1.0; Tn = 0.4   # Regler
a = 4;K = 1.0/(a*Kp*T1); Tn = T1*a**2   # Regler gemäß Sym. Optimum
R = tf([K*Tn, K],[Tn, 0])
G0=R*G
bode(G0)
gm, pm, wg, wp = control.matlab.margin(G0)
gmdb = 20*np.log10(gm); wphz = wp/(2*np.pi)
print('Reglerverstärkung = ',K,' Tn = ',Tn)
print('Amplitudenreserve = ',gm,' in dB = ',gmdb)
print('Phasenreserve = ',pm)
print('Durchtrittsfrequenz = ',wp,' in Hertz = ',wphz)

Gw = control.feedback(G0,1)
x, t = control.matlab.step(Gw)

Gy = control.feedback(R,G) # Stellgröße
y, tt = control.matlab.step(Gy)

fig, ax1 = plt.subplots()

ax1.plot(t, x, "b") 
ax1.set_ylabel('x', fontsize=14, color="blue") 
plt.grid()
ax2 = ax1.twinx() 
ax2.plot(tt, y, "r") 
ax2.set_ylabel('y', fontsize=14,color="red") 
ax1.set_xlabel('t [s]', fontsize=12) 
plt.title('Entwurf gemäß symm. Optimum')    
#Entwurf Vorfilter
vt1=0.5 ; vt2=0.5
Gvg = tf(1,[vt1*vt2, vt1+vt2, 1]) # Vorgabe
F = Gvg/Gw
print(F)