# -*- coding: utf-8 -*-
"""
Smith-Prädiktor 
Created on 13.6.23
@author: philippsen
"""

import numpy as np
import control as ct
import matplotlib.pyplot as plt
#plt.rcParams['pdf.fonttype'] = 42
#plt.rcParams['ps.fonttype'] = 42
  
pt = ct.tf(1,[1,1]) # Strecke 
ptm = ct.tf(1,[1,1]) # Modell der Strecke

K=2; Tn = 1.1
pi = ct.tf([K*Tn,K],[Tn,0])     #PI-Regler
t0 = 0.1  # Abtastzeit
totzeit = 1 # Sekunde
# Totzeit als zeitdiskretes System
nn = int(totzeit/t0)
z=ct.tf('z')
tot=(z**(-nn) )/1
tot.input_labels = 'ys' ; tot.output_labels = 'xs'
totM = (z**(-nn) )/1
totM.input_labels = 'ym' ; totM.output_labels = 'xm'

# Diskretisierung Strecke Regler
ptdisk = ct.sample_system(pt,t0,method='zoh', name='ptd',inputs='y', outputs='ys')	
ptdiskM = ct.sample_system(pt,t0,method='zoh', name='ptdM',inputs='y', outputs='ym')
pidisk = ct.sample_system(pi,t0,method='zoh', inputs='e', outputs='y')

sumblk = ct.summing_junction(inputs=['r', '-xx'], output='e')
sumblk2 = ct.summing_junction(inputs=['xs', '-xm'], output='em')
sumblk3 = ct.summing_junction(inputs=['ym', 'em'], output='xx')

Rk = ct.interconnect([tot, ptdisk,totM, ptdiskM,pidisk,
                      sumblk,sumblk2,sumblk3], inplist='r', outlist=['xs','y'])

t=np.arange(0,10,t0)
ww = np.ones(len(t))
t, yy = ct.input_output_response(Rk, t, ww)
#t1, y = ct.step_response(tot*ptdisk,t)
plt.figure(1)
plt.plot(t,yy[0,:],t,yy[1,:])
plt.ylabel('y (rot) , x (blau)')
plt.xlabel('t [s]')
plt.title('Sprungantwort Regelkreis mit Smith Prädiktor')
plt.grid(True)

