# -*- coding: utf-8 -*-
"""
Erzeugung eines Mehrgrößensystems mit der
Python Control Toolbox und Berechnung der statischen Entkopplung.
Mehrgrößensystem und PI-Regler slycot erforderlich
1. conda config --append channels conda-forge
2. conda install -c repa slycot

statische Entkopplung funktioniert nicht im Fall
integralwirkende Strecke!

Created on 22.4. 2019
@author: philippsen
"""

import numpy as np
import control
from  control.matlab import *
import matplotlib.pyplot as plt

#Gii(s) = 2/2s^2 + 3s +1    Gij = 0.5/2s+1
n11= [2.0]; d11= [2., 3., 1.]
n22= [2.0]; d22= [2., 3., 1.]
n12= [0.5]; d12= [2., 1.]
n21= [0.5]; d21= [2., 1.]
g11 = tf(n11,d11); g110 = dcgain(g11)
g12 = tf(n12,d12); g120 = dcgain(g12)
g22 = tf(n22,d22); g220 = dcgain(g22)
g21 = tf(n21,d21); g210 = dcgain(g21)
        
num = [[n11, n12], [n21, n22]]
den = [[d11, d12], [d21, d22]]
strecke = tf(num, den)
k= g110*g220/(g110*g220 - g210*g120)

ne = [[[k], [-k*g120/g110]], [[-k*g210/g220], [k]]]
de = [[[1.], [1.]], [[1.], [1.]]]
ent = tf(ne, de)
y, t = control.matlab.step(strecke)
# Plot zeigt Wirkung von Eingang 1 auf beide Ausgänge
# neu ab 0.9
plt.plot(t,y[:,0],t,y[:,1])
plt.title('Mehrgrößensystem')
plt.xlabel('t [s]'); plt.ylabel('h(t)')
plt.grid()
plt.figure()
yy, tt = control.matlab.step(ent*strecke)
# Plot zeigt Wirkung von Eingang 1 auf beide Ausgänge
plt.plot(tt,yy[:,0],tt,yy[:,1])
plt.title('statisch entkoppeltes Mehrgrößensystem')
plt.xlabel('t [s]'); plt.ylabel('h(t)')
plt.grid()
plt.show()