# -*- coding: utf-8 -*-
"""
Kran  im Zustandsraum  Reglerentwurf mit Ackermann
Created on 24.8. 2021
@author: philippsen
"""

import numpy as np
import control
from   control.matlab import *
import matplotlib.pyplot as plt
plt.rcParams['pdf.fonttype'] = 42
plt.rcParams['ps.fonttype'] = 42

T = 0.1 ; l = 0.5 ; g = 9.81
  
Am  = np.array([[0, 1,  0, 0],
                [0, -1/T, 0, 0],
                [0, 0, 0, 1],
                [0, 1/(l*T), -g/l, 0]])
bm  = np.array([[0],[ 1/T],[0],[-1/(l*T)]])
#cm  = np.array([ 0, 1])
C  = np.eye(4)   # Falls alle Zustände ausgegeben werden sollen
D  = np.array([[ 0. ],[0],[0],[0]])
linKran = ss(Am, bm, C, D)

print('Pole : ',pole(linKran))
# Zustandsreglerentwurf
pol  = np.array([-1,-1,-1,-1])
rr = control.acker(Am,bm,pol*2.0)

# nicht lineares Modell Kran
def kran_update(t, x, u, params={}):
    # Reglerparameter übergeben, falls erforderlich
    kp = params.get('kp', 0.05)
    T1 = params.get('T1', 0.1)
    l = params.get('l', 0.5)

    # Zuweisung von Variablen und Zuständen
    vsoll = u[0]          #  Vorgabe v
    lage = x[0]      # Zustand Lage
    v = x[1]      # Zustand v = Lage punkt
    alpha = x[2]      # Zustand alpha
    alphap = x[3]      # Zustand erste Ableitung alpha
    
    # Berechnung Ableitungen
    lagep = v
    vp = -v/T1 + vsoll*kp/T1
    alphapp = -np.sin(alpha)*9.81/l - vp*np.cos(alpha)/l # zweite Abl. alpha
    return [lagep, vp, alphap , alphapp]


io_kran = control.nlsys(
    kran_update, None, inputs=['u'], 
    outputs=['lage','v','alpha','alphap'],
    states=['lage','v', 'alpha', 'alphap'], name='kran',
    params = { 'kp':1.0})


def x_rueck(t, x, u, params={}):
    # parameter übergeben, falls erforderlich
    vor = params.get('vor', 1)
    # Zuweisung von Variablen und Zuständen
    x0 = u[0]; x1 = u[1]; x2 = u[2]; x3 = u[3]
    # Rückführung berechnen später Skalarprodukt
    return rr[0,0]*x0 + rr[0,1]*x1 + rr[0,2]*x2 + rr[0,3]*x3

rueck = control.nlsys(
    None, x_rueck, name='rueck',
    inputs = ['lage','v','alpha','alphap'], outputs = ['rue'],
    params = {'vor':1}) 
 
sumblk = control.summing_junction(inputs=['w', '-rue'], 
                                  output=['u'])
#Regelkreis aufbauen
Rk = control.interconnect([io_kran, rueck, sumblk], 
            inplist=['w'], outlist=['lage','alpha','u'])

T = np.arange(0.0, 10.0, 0.01) # Simulationsdauer 
ww = np.ones([len(T)])

t, yy = control.input_output_response(Rk, T, rr[0,0]*ww)
#resp = control.input_output_response(Rk, T, ww) #rr[0,0]*ww)

# Umrechnung rad Grad
ur = 180/np.pi
plt.figure(1)
plt.plot(t,yy[0,:],t,ur*yy[1,:],t,yy[2,:])
plt.title('Kran mit Zustandsregler')
plt.xlabel('t [s]'); plt.ylabel('lage,  alpha,  y')    
plt.grid(True)



