# -*- coding: utf-8 -*-
"""
PI-Regler mit verbesserter AWR I-Rückführung
Created on Thu Apr 25 15:18:58 2019
@author: philippsen
"""

import numpy as np 
import control 
import matplotlib.pyplot as plt 
 
def PI_AWR_REGLER(ek,yminmax,Kp): 
    """ PI-Regler mit verbesserter AWR I-Rückführung """ 
    global ykk , yk, Kid, Tikorr 
    global yi1, ek1 
    Intkorr = (ykk - yk)/Tikorr # Korrektur I-Anteil falls Begrenzung  
    yi = yi1 + Kid*ek  +  Kid*ek1 - Intkorr # Berechnung I-Anteil   
    ek1 = ek 
    yi1=yi           
    ykk =  Kp*ek + yi # Stellgroesse = P + I  
    if ykk < -yminmax : # Begrenzung Stellgroesse  
        yk = -yminmax  
    elif ykk > yminmax :  
        yk = yminmax  
    else: 
        yk = ykk #   y ohne Begrenzung   
 
    return yk  
#   Sim mit forced response und digitalem PI-Regler 
n = 500 # Anzahl Schritte 
T0 = 0.01 # Abtastzeit 
xx = np.zeros(n)    # Regelgrößenvektor 
u = np.zeros(n)     # Stellgrößenvektor 
yyi = np.zeros(n)   # I-Anteil Stellgröße 
tk= np.arange(0,T0*n,T0)    # k-Achse 
 
DCmo = control.tf([5],[1.2, 1]) # Motormodell 1. Ordnung 
Kp = 2; Tn = 1.4        # Reglerparameter 
Kid=Kp*T0/(2*Tn)        # Ki diskretisiert 
Tikorr = 0.96*Tn/T0     # Korrekturfaktor I-Anteil 
ykk =0;  yk =0.0; yi1 = 0.0; ek1 = 0.0; Xvor = 0 # Anfangswerte 
 
w = 20.0    #Sollwert 
for i in range(1, n): 
     e = w - xx[i-1] 
     u[i] = PI_AWR_REGLER(ek=e,yminmax=10,Kp=Kp) 
     yyi[i]=yi1    
     tt, xxk , xk = control.forced_response(DCmo,[tk[i], tk[i]+T0], 
[u[i], u[i]], X0=Xvor,return_x=True) 
     xx[i] = xxk[1] 
     Xvor = xk[0,1] 
 
plt.plot(tk,xx, tk, u, tk,yyi) 
plt.xlabel('t [s]'); plt.ylabel('x, y, yi') 
plt.grid(True) 
