# -*- coding: utf-8 -*-
"""

DC-Motor mit P-PI-Lageregler im Zustandsraum und G(s)
Kaskade gemäß Faulhaber MC5005
Created on 12.4. 2019
@author: philippsen
"""

import numpy as np
import control
from   control.matlab import *
import matplotlib.pyplot as plt

Tc = 1e-3       #  Ersatzzeitkonstante Stromregelkreis
zpiJ = 47.119e-6 #	 Meine Berechnung 2pi7,5e-6
KL = 1          #  Integrierbeiwert Lagestrecke
cPsi= 0.0318    #   Datenblatt Faulhaber
Ks = cPsi/zpiJ  #  Verstärkung Teilstrecke

a= 16
KR= 1/(a*Ks*Tc) #  Verstärkung Drehzahlregler gemäß SymOpt
Tnv = a*a*Tc    #  Nachstellzeit Drehzahlregler
Tnv = 2*a*Tc    #  Nachstellzeit Drehzahlregler modifiziert
Kreib=1e-5
 
a1 = KR*Ks; a2 = a1*Tnv; a3 = Tnv; a4 = a3*Tc

null = np.roots([a4*a2, 0, (a3*a1 - a2*a2 ), 0, -a1*a1]);
wi = np.max(null)
Kkrit = (1/KL)*(a3*a2*wi**4 - a2*a1*wi**2 -a4*a1*wi**4 +a2*a1*wi**2)/( (a2**2)*(wi**2) + a1**2 )
Kv= Kkrit/40        #   Verstärkung Lageregler
   
Ar  = np.array([[0, KL, 0, 0],
                [0, -Kreib/zpiJ, cPsi/zpiJ, 0],
                [-KR*Kv/Tc, -KR/Tc, -1/Tc, 1/Tc],
                [-KR*Kv/Tnv, -KR/Tnv, 0, 0]])
br  = np.array([[ 0. ],[ 0], [KR*Kv/Tc],[KR*Kv/Tnv]])
cr  = np.array([1, 0, 0, 0])
C  = np.array([[1, 0, 0, 0],
                [0, 1, 0, 0],
                [0, 0, 1, 0],
                [0, 0, 0, 1]])
D  = np.array([[ 0. ],[ 0], [0],[0]])
DCkreis = ss(Ar, br, C, D)

y, t = control.matlab.step(DCkreis)

#plt.gcf().clear() # löscht einen alten Plot
plt.subplot(3,1,1)
plt.plot(t,y[:,0])
plt.title('DC-Motor'); plt.ylabel('Lage')
plt.xlabel('t [s]')
plt.grid(True)
plt.subplot(3,1,2)
plt.plot(t,y[:,1],'g')
plt.xlabel('t [s]'); plt.ylabel('n(t)')
plt.grid(True)
plt.subplot(3,1,3)
plt.plot(t,y[:,2],'r')
plt.xlabel('t [s]'); plt.ylabel('Strom')
plt.grid(True)

